﻿/*
   PhysicsThread Project

      Copyright (C) 2008 Bepu Entertainment LLC

      This demonstration software, excluding the BEPUphysics.dll and BEPUphysics.xml (which are handled independently by their accompanying license),
      is provided 'as-is', without any express or implied warranty.  In no event will the authors be held liable for any damages
      arising from the use of this software.

      Permission is granted to anyone to use this software for any purpose,
      including commercial applications, and to alter it and redistribute it
      freely, subject to the following restrictions:

      1. The origin of this software must not be misrepresented; you must not
         claim that you wrote the original software. If you use this software
         in a product, an acknowledgment in the product documentation would be
         appreciated but is not required.
      2. Altered source versions must be plainly marked as such, and must not be
         misrepresented as being the original software.
      3. This notice may not be removed or altered from any source distribution.

    Contact us at:
    contact@bepu-games.com
    Ross Nordby
    norbo@bepu-games.com
    Harrison Nordby
    squashwell@bepu-games.com
 */


using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using Microsoft.Xna.Framework.Input;
using Microsoft.Xna.Framework;

namespace PhysicsThread
{
    /// <summary>
    /// Simple camera class for moving around the demos area.
    /// </summary>
    public class Camera:GameComponent
    {
        /// <summary>
        /// Position of the camera in the world.
        /// </summary>
        public Vector3 position;
        /// <summary>
        /// Rotation, in radians, around the Y axis.
        /// </summary>
        public float yaw;
        /// <summary>
        /// Rotation, in radians, around the axis perpendicular to the direction defined by the yaw.
        /// </summary>
        public float pitch;
        /// <summary>
        /// Orientation quaternion representing the rotation of the camera around the Y axis.
        /// </summary>
        public Quaternion horizontalRotation;
        /// <summary>
        /// Orientation quaternion representing the rotation of the camera around the pitch axis.
        /// </summary>
        public Quaternion pitchRotation;
        /// <summary>
        /// Orientation quaternion representing the combined horizontal and pitch rotations.
        /// </summary>
        public Quaternion rotation;
        /// <summary>
        /// Orientation matrix representing the combined horizontal and pitch rotations.
        /// </summary>
        public Matrix rotationMatrix;
        /// <summary>
        /// Movement speed of the camera.
        /// </summary>
        public float speed;
        /// <summary>
        /// View frustum around the camera.
        /// </summary>
        public BoundingFrustum frustum;
        /// <summary>
        /// View matrix built off of the position and facing of the camera.
        /// </summary>
        public Matrix viewMatrix;
        /// <summary>
        /// Projection matrix to use in rendering the camera's scene.
        /// </summary>
        public Matrix projectionMatrix;
        /// <summary>
        /// Whether or not to use the graphical display framerate over the physics frequency.  Used in update.
        /// </summary>
        public bool useRenderFramerate = true;
        /// <summary>
        /// Whether or not to use the default free-flight camera controls.
        /// Set to false when using vehicles or character controllers.
        /// </summary>
        public bool useMovementControls = true;
   
        /// <summary>
        /// Creates a camera.
        /// </summary>
        /// <param name="pos">Initial position of the camera.</param>
        /// <param name="s">Speed of the camera per second.</param>
        /// <param name="p">Initial pitch angle of the camera.</param>
        /// <param name="y">Initial yaw value of the camera.</param>
        /// <param name="projMatrix">Projection matrix used.</param>
        /// <param name="game">Game owning this component.</param>
        public Camera(Vector3 pos, float s, float p, float y, Matrix projMatrix, Game game):base(game)
        {
            position = pos;
            speed = s;
            yaw = y;
            pitch = p;
            findViewMatrix();
            projectionMatrix = projMatrix;
            frustum = new BoundingFrustum(viewMatrix * projMatrix);
            Mouse.SetPosition(200, 200);
           
            
        }
        /// <summary>
        /// Computes the view matrix of the camera.
        /// </summary>
        public void findViewMatrix()
        {//TODO
            rotationMatrix = Matrix.CreateFromQuaternion(pitchRotation) * Matrix.CreateFromQuaternion(horizontalRotation);
            viewMatrix = Matrix.CreateLookAt(position, position + Vector3.Transform(Vector3.Forward, rotationMatrix), Vector3.Up);
        }
        /// <summary>
        /// Computes the vector facing forward from the camera.
        /// </summary>
        /// <returns>Vector facing forward from the camera.</returns>
        public Vector3 getForwardVector()
        {
            return rotationMatrix.Forward;
        }
        /// <summary>
        /// Computes the vector pointing backward from the camera.
        /// </summary>
        /// <returns>Vector pointing backward from the camera.</returns>
        public Vector3 getBackVector()
        {
            return rotationMatrix.Backward;
        }
        /// <summary>
        /// Computes the vector pointing to the left of the camera.
        /// </summary>
        /// <returns>Vector pointing to the left of the camera.</returns>
        public Vector3 getLeftVector()
        {
            Matrix rotMat = Matrix.CreateFromQuaternion(horizontalRotation);
            return rotMat.Left;
        }
        /// <summary>
        /// Computes the vector pointing to the right of the camera.
        /// </summary>
        /// <returns>Vector pointing to the right of the camera.</returns>
        public Vector3 getRightVector()
        {
            Matrix rotMat = Matrix.CreateFromQuaternion(horizontalRotation);
            return rotMat.Right;
        }
        /// <summary>
        /// Adjusts the facing direction of the camera.
        /// </summary>
        /// <param name="yawAngle">Change in the yaw angle to apply.</param>
        /// <param name="pitchAngle">Change in the pitch angle to apply.</param>
        public void changeRotation(float yawAngle, float pitchAngle)
        {
            yaw += yawAngle;
            pitch += pitchAngle;
            if (pitch > 1.55f)
                pitch = 1.55f;
            else if (pitch < -1.55f)
                pitch = -1.55f;
            horizontalRotation = Quaternion.CreateFromAxisAngle(Vector3.Up, yaw);
            pitchRotation = Quaternion.CreateFromAxisAngle(Vector3.Right, pitch);
            rotation = pitchRotation * horizontalRotation;
            rotationMatrix = Matrix.CreateFromQuaternion(pitchRotation) * Matrix.CreateFromQuaternion(horizontalRotation);
        }
        /// <summary>
        /// Moves the camera up and down the Y axis.
        /// </summary>
        /// <param name="distance">Distance to move along the Y axis.</param>
        public void changeAltitude(float distance)
        {
            position.Y += distance;
        }
        /// <summary>
        /// Moves the camera along its forward vector.
        /// </summary>
        /// <param name="distance">Distance to move the camera along its forward vector.</param>
        public void moveForward(float distance)
        {
            position += rotationMatrix.Forward * distance;
        }
        /// <summary>
        /// Moves the camera along its backward vector.
        /// </summary>
        /// <param name="distance">Distance to move the camera along its backward vector.</param>
        public void moveBackward(float distance)
        {
            position += rotationMatrix.Backward * distance;
        }
        /// <summary>
        /// Moves the camera along its left-pointing vector.
        /// </summary>
        /// <param name="distance">Distance to move the camera along its left-pointing vector.</param>
        public void moveLeft(float distance)
        {
            position += Matrix.CreateFromQuaternion(horizontalRotation).Left * distance;
        }
        /// <summary>
        /// Moves the camera along its right-pointing vector.
        /// </summary>
        /// <param name="distance">Distance to move the camera along its right-pointing vector.</param>
        public void moveRight(float distance)
        {
            position += Matrix.CreateFromQuaternion(horizontalRotation).Right * distance;
        }

        public override void Update(GameTime gameTime)
        {
            base.Update(gameTime);
            float dt = (float)gameTime.ElapsedRealTime.TotalSeconds;
        
#if XBOX360
        KeyboardState keyboardInput = Keyboard.GetState();
        GamePadState gamePadInput = GamePad.GetState(0);
#else
        KeyboardState keyboardInput = Keyboard.GetState();
        GamePadState gamePadInput = GamePad.GetState(0);
        MouseState mouseInput = Mouse.GetState();
       

#endif

#if XBOX360            
            changeRotation(gamePadInput.ThumbSticks.Right.X * -1.5f * (float)gameTime.ElapsedRealTime.TotalSeconds, gamePadInput.ThumbSticks.Right.Y * 1.5f * (float)gameTime.ElapsedRealTime.TotalSeconds);
#else
            changeRotation((200 - mouseInput.X) * .002f, (200 - mouseInput.Y) * .002f);
#endif
            Mouse.SetPosition(200, 200);

            if (useMovementControls)
            {//Only move around if the camera has control over its own position.
                float distance = speed * dt;
#if XBOX360
                moveForward(gamePadInput.ThumbSticks.Left.Y * distance);
                moveRight(gamePadInput.ThumbSticks.Left.X * distance);
                if (gamePadInput.IsButtonDown(Buttons.LeftStick))
                    changeAltitude(distance);
                if (gamePadInput.IsButtonDown(Buttons.RightStick))
                    changeAltitude(-distance);
#endif

                if (keyboardInput.IsKeyDown(Keys.E))
                    moveForward(distance);
                if (keyboardInput.IsKeyDown(Keys.D))
                    moveBackward(distance);
                if (keyboardInput.IsKeyDown(Keys.S))
                    moveLeft(distance);
                if (keyboardInput.IsKeyDown(Keys.F))
                    moveRight(distance);
                if (keyboardInput.IsKeyDown(Keys.A))
                    changeAltitude(distance);
                if (keyboardInput.IsKeyDown(Keys.Z))
                    changeAltitude(-distance);
            }

            findViewMatrix();
            frustum.Matrix = viewMatrix * projectionMatrix;


        }


    }
}
